// ----------------------------------------------------------------------
// termBlaster - The context search extension
//  @copyright 2006 David Duong
//  @version $Id: blaster.js,v 1.4 2006/03/01 03:11:17 davidnqd Exp $
// ----------------------------------------------------------------------
// LICENSE
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have recieved the lincence included with the disribution.
// If not visit http://www.gnu.org/copyleft/gpl.html
// ----------------------------------------------------------------------
var Blaster = {
  opened: "",
  changed: false,

  onLoad: function() {
    // Get the quick list URL
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefBranch);
    var uri = prefs.getCharPref("extensions.termblaster.opened.quicklist").replace(/^\s*|\s*$/g, "");
    if (uri.length == 0) {
      return false;
    }

    // Executes after the XML file has been loaded
    var XMLDoc = document.implementation.createDocument("", "", null);
    XMLDoc.async = false;
    XMLDoc.onload = function () {
      var iNode = XMLDoc.documentElement;
      var cNode;
      for (var i = 0; i < iNode.childNodes.length; i++) {
        cNode = iNode.childNodes.item(i);
        if (cNode != null
         && cNode.localName != null
         && cNode.localName == 'service') {
          lbSearch.appendItem(cNode.attributes.getNamedItem("name").nodeValue, cNode.attributes.getNamedItem("query").nodeValue);
        }
      }
    }

    // Loaded items will be stored here:
    var lbSearch = document.getElementById("services");

    // Load an XML file
    try {
      XMLDoc.load(uri);
    } catch (e) {
      alert("termBlaster could not load selected file ('" + uri + "'): " + e.message);
    }

    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                      getService(Components.interfaces.nsIPrefBranch);
    var uri = prefs.getCharPref("extensions.termblaster.opened.terms");
    Blaster.load(uri);
  },
  
  onUnload: function() {
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                      getService(Components.interfaces.nsIPrefBranch);
    var showConfirm = prefs.getBoolPref("extensions.termblaster.terms.showConfirm");
    if (Blaster.changed && showConfirm) {
      alert('Changes to the terms list have not been saved, and were lost. In the next release, this will be a "do you wish to save.." dialog box.');
    }
  },

  onClose: function() {
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                      getService(Components.interfaces.nsIPrefBranch);
    var showConfirm = prefs.getBoolPref("extensions.termblaster.terms.showConfirm");
    if (Blaster.changed && showConfirm) {
      var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
        .getService(Components.interfaces.nsIPromptService);
      var flags=promptService.BUTTON_TITLE_OK * promptService.BUTTON_POS_0 +
          promptService.BUTTON_TITLE_NO * promptService.BUTTON_POS_1 +
          promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_2;
      var checkResult = {};
      var result = promptService.confirmEx(window,"Save terms list?",
        "The terms list has been changed, do you wish to save?", flags, null, null, null,
        "Don't show this message again (Always don't save)", checkResult);
      if (checkResult.value) {
        prefs.setBoolPref("extensions.termblaster.terms.showConfirm", false);
      }
      if (result == 0) {
        Blaster.onTermsSaveClick();
      } else if (result == 2) {
        return false
      }
    }
    return true;
  },

  onKeyPress: function(event) {
    if (event.keyCode == event.DOM_VK_RETURN) {
      Blaster.add(document.getElementById("search").value);
      document.getElementById("search").value = '';
    }
  },

  onChangeSelection: function() {
    // Get the selected term
    // If none is selected, then the first is assumed
    if (document.getElementById("terms").selectedItem != null) {
      var term = document.getElementById("terms").selectedItem.value.replace(/^\s*|\s*$/g,"");
    } else if (document.getElementById("terms").getItemAtIndex(0) != null) {
      var term = document.getElementById("terms").getItemAtIndex(0).value.replace(/^\s*|\s*$/g,"");
    } 

    // If term is defined, then return
    if (typeof(term) == "undefined") {
      return;
    }

    // Get the query in a similiar fasion as the term
    if (document.getElementById("services").selectedItem != null) {
      var query = document.getElementById("services").selectedItem.value.replace('\$s', encodeURIComponent(term));
    } else if (document.getElementById("services").getItemAtIndex(0) != null) {
      var query = document.getElementById("services").getItemAtIndex(0).value.replace('\$s', encodeURIComponent(term));
    }

    // If query is empty then return
    if (typeof(query) == "undefined" && query.length == 0) {
      return;
    }

    // Load the results in the current window
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                       .getService(Components.interfaces.nsIWindowMediator);
    var win = wm.getMostRecentWindow("navigator:browser");
    win.getBrowser().loadURI(query);
  },

  onTermsClearClick: function() {
    Blaster.changed = true;

    Blaster.clear();
  },

  onTermsNewClick: function() {
    Blaster.onChangeFile('');
    Blaster.clear();
    Blaster.changed = false;
  },

  onTermsSaveClick: function() {
    if (Blaster.opened == "") {
      Blaster.onTermsSaveAsClick();
    } else {
      Blaster.save(Blaster.opened);
    }
  },

  onTermsSaveAsClick: function() {
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

    fp.init(window, "", nsIFilePicker.modeSave);
    //fp.appendFilter("termBlaster Terms Files", "*.tbt");
    //fp.appendFilters(nsIFilePicker.filterText);
    fp.appendFilters(nsIFilePicker.filterAll);

    var dialog = fp.show();

    if (dialog == nsIFilePicker.returnOK || dialog == nsIFilePicker.returnReplace) {
      Blaster.save(fp.file);
    }
  },
  
  onTermsOpenClick: function() {
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

    fp.init(window, "", nsIFilePicker.modeOpen);
    //fp.appendFilter("termBlaster Terms Files", "*.tbt");
    //fp.appendFilters(nsIFilePicker.filterText);
    fp.appendFilters(nsIFilePicker.filterAll);

    if (fp.show() == nsIFilePicker.returnOK && fp.fileURL.spec && fp.fileURL.spec.length > 0) {
      Blaster.load(fp.file);
    }
  },
  
  onChangeFile: function(file) {
    Blaster.opened = file;
    if (file != null) {
      var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                      getService(Components.interfaces.nsIPrefBranch);
      prefs.setCharPref("extensions.termblaster.opened.terms", file);
    }
  },

  add: function(term) {
    term = term.replace(/^\s*|\s*$/g,"");
    if (term != '') {
      Blaster.changed = true;
      document.getElementById("terms").appendItem(term, term);
    }
  },

  clear: function (file) {
    Blaster.saved = true;

    var terms = document.getElementById('terms');
    var rowCount = terms.getRowCount();
    for(var i = 0; i < rowCount; i++)	{
      terms.removeItemAt(0);
    }
  },

  save: function(file) {
    if (typeof file == 'string') {
      var file_name = file.replace(/^\s*|\s*$/g,"");
      if (file_name == '') {
        return;
      }
      try {
        var file = Components.classes["@mozilla.org/file/local;1"]
                             .createInstance(Components.interfaces.nsILocalFile);
        file.initWithPath(file_name);
      } catch (e) {
        alert ("Could not open the terms file (" + file_name + ")");
        return;
      }
    }
    if (Blaster.changed) {
      try {
        var terms = document.getElementById('terms');
        // open an output stream from file
        var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                           .createInstance(Components.interfaces.nsIFileOutputStream);
        // use 0x02 | 0x10 to open file for appending.
        foStream.init(file, 0x02 | 0x08 | 0x20, 0664, 0); // write, create, truncate
        var rowCount = terms.getRowCount();
        // write each listitem
        var data = '';
        for(var i = 0; i < rowCount; i++) {
          data = terms.getItemAtIndex(i).label + '\n';
          foStream.write(data, data.length);
        }
        foStream.close();
        Blaster.onChangeFile(file.path);
        Blaster.changed = false;
      } catch (e) {
        alert ("Could not save the terms file (" + file.path + "): " + e);
      }
    }
  },

  load: function(file) {
    if (typeof file == 'string') {
      var file_name = file.replace(/^\s*|\s*$/g,"");
      if (file_name == '') {
        return;
      }
      try {
        var file = Components.classes["@mozilla.org/file/local;1"]
                             .createInstance(Components.interfaces.nsILocalFile);
        file.initWithPath(file_name);
      } catch (e) {
        alert ("Could not open the terms file (" + file_name + ")");
        return;
      }
    }
    var terms = document.getElementById('terms');
    // Get rid of whatever was loaded before
    Blaster.clear();

    // open an input stream from file
    var fiStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                          .createInstance(Components.interfaces.nsIFileInputStream);
    try {
      fiStream.init(file, 0x01, 0444, 0);
      fiStream.QueryInterface(Components.interfaces.nsILineInputStream);
  
      // read lines into array
      var line = {}, lines = [], cont;
      do {
        cont = fiStream.readLine(line);
        Blaster.add(line.value);
      } while(cont);
      Blaster.onChangeFile(file.path);
      Blaster.changed = false;
    } catch (e) {
        alert ("Could not read the terms file (" + file.path + "): " + e);
    } finally {
      fiStream.close();
    }
  }
}
